// Created on: 1992-02-11
// Created by: Christian CAILLET
// Copyright (c) 1992-1999 Matra Datavision
// Copyright (c) 1999-2014 OPEN CASCADE SAS
//
// This file is part of Open CASCADE Technology software library.
//
// This library is free software; you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License version 2.1 as published
// by the Free Software Foundation, with special exception defined in the file
// OCCT_LGPL_EXCEPTION.txt. Consult the file LICENSE_LGPL_21.txt included in OCCT
// distribution for complete text of the license and disclaimer of any warranty.
//
// Alternatively, this file may be used under the terms of Open CASCADE
// commercial license or contractual agreement.

#include <StepData_FileRecognizer.hxx>

#include <Standard_Type.hxx>

#include <Standard_Transient.hxx>
#include <StepData_FileRecognizer.hxx>
#include <Standard_NoSuchObject.hxx>
#include <TCollection_AsciiString.hxx>

 








#define TheKey TCollection_AsciiString
#define TheKey_hxx <TCollection_AsciiString.hxx>
#define Handle_TheResul Handle(Standard_Transient)
#define TheResul Standard_Transient
#define TheResul_hxx <Standard_Transient.hxx>
#define Interface_Recognizer StepData_FileRecognizer
#define Interface_Recognizer_hxx <StepData_FileRecognizer.hxx>
#define Handle_Interface_Recognizer Handle(StepData_FileRecognizer)
#include <Interface_Recognizer.gxx>

