<Application
  x:Class="uwp.App"
  xmlns="http://schemas.microsoft.com/winfx/2006/xaml/presentation"
  xmlns:x="http://schemas.microsoft.com/winfx/2006/xaml"
  xmlns:local="using:uwp"
  RequestedTheme="Light">
</Application>
